/*  Project RBL
	 R.B. Logiciel
	 Copyright  1994..1995 All Rights Reserved.
	 SUBSYSTEM:    pmdi.exe Application
	 FILE:         editdate.cpp
	 AUTHOR:       R. BERTHOU 	 (CIS : 100654,3225)


	 OVERVIEW
	 ========
	 Source file for implementation of TEditDate (TEdit).
*/
#define RBL_US_    	//For US Format

#include <string.h>
#include <stdlib.h>
#include	<ctype.h>
#include "divers.h"
#include "editdate.h"
#include "datedlg.h"

DEFINE_RESPONSE_TABLE1(RButton, TButton)
	EV_NOTIFY_AT_CHILD(BN_CLICKED, EvClicked),
END_RESPONSE_TABLE;

RButton::RButton(TWindow*        parent,
					  int             id,
					  const char far* text,
					  TEditDate*		edt,
					  int x, int y, int w, int h,
					  BOOL            isDefault,
					  TModule*        module)
  : TButton(parent, id, text, x, y, w, h, isDefault, module)
{
	Attr.Style &= ~WS_TABSTOP;
	EDt = edt;
}


void
RButton::EvClicked()
{
	EDt->CmCal();
}

DEFINE_RESPONSE_TABLE1(TEditDate, TEdit)
 EV_WM_CHAR,
 EV_WM_SETFOCUS,
 EV_WM_KILLFOCUS,
END_RESPONSE_TABLE;

TEditDate::TEditDate(TWindow *Parent,
							int 		Id,
							TDate 	dtT,
							UINT 		textLen,
							TModule* module)
			 : TEdit	  (Parent, Id,	textLen, module)
{
	dt = dtT ;
	dtDlg = new DateDlg(this) ;
	SetValidator(new TDateValidator() ) ;
}

TEditDate::TEditDate(TWindow* parent,	int id,
							 int x, int y, int w, int h,
							 TDate dtT,
							 uint  textLen,
							 bool  multiline,
							 TModule*        module)
			 : TEdit	  (parent, id,	"", x, y, w, h, textLen, multiline, module)

{
	dt = dtT ;
	SetValidator(new TDateValidator() ) ;
	dtDlg = new DateDlg(this) ;
}

TEditDate::~TEditDate()
{
	delete dtDlg ;
	if (bCal)
		delete bCal ;
}

void
TEditDate::SetupWindow()
{
	bCal = new RButton(Parent, 0, "", this,
												 Attr.X + Attr.W ,
												 Attr.Y,
												 Attr.H * 2 / 3,
												 Attr.H) ;
	dtDeb = TDate(-1,-1) ;
	TEdit::SetupWindow() ;
}

void
TEditDate::SetText(const char far *buffer)
{
	int iJour, iMois, iAnnee ;

	if (strlen(buffer) > 0) {
		ConvDate((char *) buffer, iJour, iMois, iAnnee) ;
		if (TDate(iJour, iMois, iAnnee).IsValid())
			SetText(TDate(iJour, iMois, iAnnee)) ;
	}
	else {
		dt = TDate(0, 0) ;
		TEdit::SetText("") ;
	}
}

void
TEditDate::SetText(TDate dtTemp)
{
	if (dtTemp.IsValid())
		dt = dtTemp ;

	if ((dtTemp.Day() == 0) && (dtTemp.Year() == 0))
		TEdit::SetText("") ;
	else
		TEdit::SetText(dt.AsString().c_str()) ;
}


void
TEditDate::SetText(int iJ, int iM, int iA)
{
	if ((iJ != 0) &&  (TDate(iJ, iM, iA).IsValid()) )
			SetText(TDate(iJ, iM, iA)) ;
	else {
			dt = TDate(0, 0);
			TEdit::SetText("") ;
	}
}


void
TEditDate::GetText(int &iJ, int &iM, int &iA)
{
	if (dt == TDate(0,0) )
		iJ = iM = iA = 0;
	else {
		iJ = dt.DayOfMonth();
		iM = dt.Month();
		iA = dt.Year();
	}
}




void
TEditDate::EvChar(UINT key, UINT repeatCount, UINT flags)
{
	switch (key)
	{
	 case '+' :                                                    // '+'
		dt += repeatCount;
		SetText(dt) ;
		break;
	 case '-' :                                                    // '-'
		dt -= repeatCount;
		SetText(dt) ;
		break;
	 case '*' :                                                    // '*'
		dt = TDate();
		SetText(dt) ;
		break;
	 case 'C' :
	 case 'c' :
		CmCal() ;
		break;
	 default :
		if (isalpha(key) == 0)
			TEdit::EvChar(key, repeatCount, flags);
	 }
}

void
TEditDate::EvKillFocus(HWND hWndGetFocus)
{
	GetDate() ;
	TEdit::EvKillFocus(hWndGetFocus) ;
}

void
TEditDate::GetDate(int iType)
{
	int iJour, iMois, iAnnee ;
	char buffer[11] = "";

	GetText(buffer, 11) ;
	if (strlen(buffer) > 0) {
		ConvDate((char *) buffer, iJour, iMois, iAnnee) ;
		if (iType == 0)
			SetText(iJour, iMois, iAnnee) ;
	}
	else {
		dt = TDate(0, 0) ;
		if (iType == 0)
			TEdit::SetText("") ;
	}

	if (dt != dtDeb)
		SetModify() ;
}

void
TEditDate::EvSetFocus(HWND hWndLostFocus)
{
	int iJour, iMois, iAnnee ;
	char buffer[11] = "";

	GetText(buffer, 11) ;
	if (strlen(buffer) > 0) {
		ConvDate((char *) buffer, iJour, iMois, iAnnee) ;
		dt = TDate(iJour, iMois, iAnnee) ;
		if (dtDeb == TDate(-1,-1))
			dtDeb = dt ;
	}
	else {
		if (dtDeb == TDate(-1,-1))
			dtDeb = TDate(0,0) ;
		dt = TDate() ;
	}


	TEdit::EvSetFocus(hWndLostFocus) ;
}

void
TEditDate::CmCal ()
{
	EvSetFocus(0) ;
	dtDlg->Create() ;
}

//----------------------------------------------------------------------------

TDateValidator::TDateValidator(const int /*format*/)
  : TValidator()
{

}

BOOL
TDateValidator::IsValid(const char far* str)
{
	int iJour = 0, iMois = 0, iAnnee = 0;
	TDate	dTemp ;

	if (strlen(str) > 0) {
		ConvDate((char *)str, iJour, iMois, iAnnee) ;
		dTemp = TDate(iJour, iMois, iAnnee) ;
		return dTemp.IsValid() ;
	}
	else
		return TRUE ;
}

void
TDateValidator::Error()
{

#if defined RBL_US_
	MessageBox(NULL,"Invalide Date\n"
					  "(MM DD AAAA  MM/DD/YY MMDDYY ...)",
							  "Error", MB_OK | MB_ICONINFORMATION) ;
#else
	MessageBox(NULL,"Le Format de la date \n est invalide\n"
					  "(JJ MM AAAA  JJ/MM/AA JJMMAA...)",
							  "Erreur", MB_OK | MB_ICONINFORMATION) ;
#endif
}


//////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////
void ConvDate(char * strDate,	// format JJ/MM/AA
				  int&  iJour,
				  int&  iMois,
				  int&  iAnnee )
{
 int iType = 0;
 int i=0;
 int j=0;
 char	pp[15] ;
 char	p[5] ;
 iJour = 0;
 iMois = 0;
 iAnnee = 0;

	strcpy(pp,strDate) ; p[2] = 0 ; pp[14]=0 ;

	while ((pp[i] != 0) && (iAnnee==0))
	{
		if (pp[i]>='0' && pp[i]<='9')
			if (iType<2 && j==2) {
				p[j] = 0 ;
				if (iType == 0)
#if defined RBL_US_
					 iMois	= atoi(p) ;
#else
					 iJour	= atoi(p) ;
#endif

				else
#if defined RBL_US_
					 iJour	= atoi(p) ;
#else
					 iMois	= atoi(p) ;
#endif
				iType++;
				j = 0;
			}
			else
				p[j++] = pp[i++] ;
		else {
			p[j] = 0 ;
			i++;
			if (iType == 0)
#if defined RBL_US_
			 iMois	= atoi(p) ;
#else
			 iJour	= atoi(p) ;
#endif
			else
				if (iType == 1)
#if defined RBL_US_
					 iJour	= atoi(p) ;
#else
					 iMois	= atoi(p) ;
#endif
				else
					 iAnnee	= atoi(p) ;
			if (j > 0)
				iType++;
			j = 0;
		}
	}
	if (iAnnee ==0)
		iAnnee = atoi(p) ;

	 if (iAnnee < 100)
		iAnnee += 1900 ;

}


